#!/bin/bash
# ------------------------------------------------------------------------------
# (C) Copyright 2022 Micro Focus or one of its affiliates.
#
# The only warranties for products and services of Micro Focus and its 
# affiliates and licensors (“Micro Focus”) are as may be set forth in the 
# express warranty statements accompanying such products and services. 
# Nothing herein should be construed as constituting an additional warranty. 
# Micro Focus shall not be liable for technical or editorial errors or 
# omissions contained herein. The information contained herein is subject 
# to change without notice.
#
# Except as specifically indicated otherwise, this document contains confidential 
# information and a valid license is required for possession, use or copying. If 
# this work is provided to the U.S. Government, consistent with FAR 12.211 and 
# 12.212, Commercial Computer Software, Computer Software Documentation, and 
# Technical Data for Commercial Items are licensed to the U.S. Government under
# vendor's standard commercial license.
# ------------------------------------------------------------------------------

LDAP_SEARCH_BIN="env LDAPTLS_CACERT=/etc/opt/novell/certs/SSCert.pem /usr/bin/ldapsearch"
LDAP_SEARCH_BIN_NOSSLPORT="/usr/bin/ldapsearch"
LDAP_SEARCH_BIN_NOCERT="env LDAPTLS_CACERT=/etc/opt/novell/certs/SSCert.pem LDAPTLS_REQCERT=never /usr/bin/ldapsearch"
LDAP_MOD_BIN="env LDAPTLS_CACERT=/etc/opt/novell/certs/SSCert.pem /usr/bin/ldapmodify"
UPDATE_LDIF_FILE="/tmp/modncpserver.ldif"
NDSTRACE="/opt/novell/eDirectory/bin/ndstrace"
LOG_FILE="/var/opt/novell/log/ncpservobjupdate.log"
LDAP_LISTENERS_FILE="/var/opt/novell/eDirectory/data/ldap_listeners.txt"
NDSCONF="/etc/opt/novell/eDirectory/conf/nds.conf"

#Before running this script, admin password should be exported using OES_ADMIN_DATA environment variable

#Check for root access
user_id=`id -u`
if test $user_id -ne 0
then
	echo "ncp_update_cached_attrs_on_ext_refs.sh: Only root user can run this script"
	exit 1
fi

Usage()
{
	echo ""
	echo "Usage: ncp_update_cached_attrs_on_ext_refs.sh [options]"
	echo "          -h                      Prints this summary"
	echo "          -d                      LDAP Admin FDN"
	echo "          -i                      LDAP Server IP address"
	echo "          -p                      LDAP Port"
	echo "          -s                      SSL or Not(1 or 0). Default 1"
	echo ""
	exit 1;
}

#Read the command line arguments and store them
while getopts d:i:p:u:s:h o
do
	case "$o" in
		d)LDAP_ADM="$OPTARG";;
		i)LDAP_IP="$OPTARG";;
		p)LDAP_PORT="$OPTARG";;
		s)SSLORNOT="$OPTARG";;
		h)Usage;;
		[?]) Usage;;
	esac
done

if test "$LDAP_ADM" = "" || test "$LDAP_IP" = "" || test "$LDAP_PORT" = "" || test "$SSLORNOT" = "" 
then
	Usage
fi

if [ -n "$OES_ADMIN_DATA" ]; then
	OES_DATA="$OES_ADMIN_DATA"
fi

logindn=$LDAP_ADM
password="$OES_DATA"
ssl=$SSLORNOT
hostname=$LDAP_IP
port=$LDAP_PORT
localLDAPIP="127.0.0.1"
localLDAPSIP="127.0.0.1"
localLDAPPort="389"
localLDAPSPort="636"

#fetch local ldap IPs from ldap_listeners.txt
fetch_local_ldap_ip_port()
{
  if [ -f "$LDAP_LISTENERS_FILE" ]
  then
        ldapIPPort=`awk '{if ($1 == "n4u.ldap.interfaces.ldap") print $3}' "$LDAP_LISTENERS_FILE"`
        if [ ! -z ldapIPPort ]
        then
             IFS=':' read -r -a ipArray <<< "$ldapIPPort"
             if  [ ! -z ${ipArray[1]} ] && [ ${ipArray[0]} != "0.0.0.0" ]
             then
                  localLDAPIP=${ipArray[0]}
                  localLDAPPort=${ipArray[1]}
             fi
        fi
        ldapsIPPort=`awk '{if ($1 == "n4u.ldap.interfaces.ldaps") print $3}' "$LDAP_LISTENERS_FILE"`
        if [ ! -z ldapsIPPort ]
        then
             IFS=':' read -r -a ipArray <<< "$ldapsIPPort"
             if [ ! -z ${ipArray[1]} ] && [ ${ipArray[0]} != "0.0.0.0" ]
             then
                  localLDAPSIP=${ipArray[0]}
                  localLDAPSPort=${ipArray[1]}
             fi
        fi
  fi
}

update_cached_attrs_on_ext_refs() {

	fetch_local_ldap_ip_port

	if [ $ssl != 0 ]
	then
		NCP_Server_Object=`$LDAP_SEARCH_BIN_NOCERT  -H ldaps://$localLDAPSIP:$localLDAPSPort -D "$logindn" -w "$password" -b "" -x -s base dsaName -LLL | sed ' /^ / {; H; d; }; /^ /! {; x; s/\n //g; }; ' | cut -c 10- | grep -vE '^$'`
	else
		NCP_Server_Object=`$LDAP_SEARCH_BIN_NOSSLPORT -H ldap://$localLDAPIP:$localLDAPPort -D "$logindn" -w "$password" -b "" -x -s base dsaName -LLL | sed ' /^ / {; H; d; }; /^ /! {; x; s/\n //g; }; ' | cut -c 10- | grep -vE '^$'`
	fi

	if [ "x" == "x$NCP_Server_Object" ]
	then
		echo "Cannot find NCP Server Object." >> $LOG_FILE
		exit 2
	fi

	if [ $ssl != 0 ]
	then
		currentCAER=`$LDAP_SEARCH_BIN -H ldaps://$hostname:$port -D "$logindn" -w "$password" -b "$NCP_Server_Object"  -s base -x -LLL CachedAttrsOnExtRefs | sed 1d | cut -c 23-`
	else
		currentCAER=`$LDAP_SEARCH_BIN -H ldap://"$hostname":$port -D "$logindn" -w "$password" -b "$NCP_Server_Object" -s base -x -LLL  CachedAttrsOnExtRefs | sed 1d | cut -c 23-`
	fi

	if [ "x" == "x$currentCAER" ]
	then
		echo "CachedAttrsOnExtRefs attribute is not exist in $NCP_Server_Object" >> $LOG_FILE
		exit 2
	else
		currentOC=`echo "$currentCAER" | grep -i "object class"`
		if [ "x" != "x$currentOC" ]
		then
                        echo -e "Object Class attribute exist in CachedAttrsOnExtRefs attribute" >> $LOG_FILE
                        exit 0
		else
			echo -e "Adding Object Class attribute to CachedAttrsOnExtRefs \nNCP Server Object : $NCP_Server_Object" >> $LOG_FILE
			mkdir -p /tmp
			touch $UPDATE_LDIF_FILE

			echo "dn: $NCP_Server_Object" > $UPDATE_LDIF_FILE
			echo "changetype: modify" >> $UPDATE_LDIF_FILE
			echo "add: CachedAttrsOnExtRefs" >> $UPDATE_LDIF_FILE
			echo "CachedAttrsOnExtRefs: Object Class" >> $UPDATE_LDIF_FILE

			if [ $ssl != 0 ]
			then
				var=`$LDAP_MOD_BIN -x -c -a -H ldaps://"$hostname":$port -D "$logindn"  -w "$password" -f $UPDATE_LDIF_FILE ` >>$LOG_FILE 2>&1
			else
				var=`$LDAP_MOD_BIN -x -c -a -H ldap://"$hostname":$port -D "$logindn"  -w "$password" -f  $UPDATE_LDIF_FILE` >>$LOG_FILE 2>&1
			fi
			Ret=$?
			echo -e "[ncp_update_cached_attrs_on_ext_refs.sh] update_cached_attrs_on_ext_refs function Completed . Return Value : $Ret" >> $LOG_FILE
			rm -f $UPDATE_LDIF_FILE

			if [ $Ret == 0 ]
			then
				sed -i '/n4u.nds.backlink-interval/d' $NDSCONF &> /dev/null
				echo -e "n4u.nds.backlink-interval=60" >> $NDSCONF
			fi
		fi
	fi
}

serverContext=`/opt/novell/eDirectory/bin/ndsconfig get | grep n4u.nds.server-context | tr -d ' ' | cut -d"=" -f2- | tr . ,`
echo "Server Context is: $serverContext" >> $LOG_FILE

update_cached_attrs_on_ext_refs

unset OES_DATA
exit 0