#!/usr/bin/python3

import subprocess as spr
import os, sys
import getopt
import logging
import xml.etree.ElementTree as et
from stat import *
from os import path
#encoding=utf8
import sys

def ParseTrusteeInfo(vol):
	if debug:
		logging.info("Parsing TrusteeInfo of Volume: " + vol)
	try:
#		command = "cat /_admin/Manage_NSS/Volume/"+vol.strip()+"/TrusteeInfo.xml | sed -e 's#<name>#<name><![CDATA[#' -e 's#</name>#]]></name>#' -e 's#<path>#<path><![CDATA[#' -e 's#</path>#]]></path>#'"
		command = "cat /_admin/Manage_NSS/Volume/"+vol.strip()+"/TrusteeInfo.xml"
		p = spr.Popen(command, shell=True, stdout=spr.PIPE, stderr=spr.PIPE, stdin=spr.PIPE)
		output = p.stdout.read()
		err = p.stderr.read()
		if not err == '':
			logging.error("Error Parsing TrusteeInfo of Volume: " + vol)
			sys.exit(0)
		trinfo = et.fromstring(output)
		return trinfo
	except Exception as e:
		logging.error("Error Parsing TrusteeInfo of Volume: " + vol + " Error: " + str(e))
		sys.exit(0)

def SetTrustee(path, tname, rights, type):
	if type == "adID":
		setcmd = ["rights", "-a","-f", path, "-r", rights , "trustee", tname] 
	else:
		setcmd = ["rights", "-f",path, "-r", rights , "trustee", tname] 
	logging.debug("Executing command: " + ' '.join([str(elem) for elem in setcmd]))
	proc = spr.Popen(setcmd, stdout=spr.PIPE, stderr=spr.PIPE)
	res = proc.communicate()
	if proc.returncode == 0:
		logging.info("Trustee Modified for Path " + path + " Trustee: " + tname + " Rights " + rights)
	if proc.returncode != 0:
		logging.error("Command: " + ' '.join([str(elem) for elem in setcmd]))
		logging.error("Output: " + res[0])
		logging.error("Error: " + res[1].replace('\n','\t'))
	
	
def SetIRF(path, rights):
	setcmd = ["rights", "-f", path, "-r", rights, "irf"] 
	logging.debug("Executing command: " + ' '.join([str(elem) for elem in setcmd]))
	proc = spr.Popen(setcmd, stdout=spr.PIPE, stderr=spr.PIPE)
	res = proc.communicate()
	if proc.returncode == 0:
		logging.info("IRF Modified for Path " + path + " Value " + rights)
	else:
		logging.error("Command: " + ' '.join([str(elem) for elem in setcmd]))
                logging.error("Output: " + res[0])
                logging.error("Error: " + res[1].replace('\n','\t'))
	

def FindPath(sec_trinfo, pr_path,sec_mountpath):
        for sec_file in sec_trinfo[0]:
                for sec_finfo in sec_file:
			if long_name_space:
				if sec_finfo.tag == "path" and sec_finfo.text.lower() == pr_path.lower():
					return True, sec_file, True
			else:
				if sec_finfo.tag == "path" and sec_finfo.text == pr_path:
                                        return True, sec_file, True
	cmd = ["ls", "-ld", sec_mountpath + pr_path ]
	proc = spr.Popen(cmd, stdout=spr.PIPE, stderr=spr.PIPE)
	res = proc.communicate()
        if proc.returncode == 0:
		return False, [], True
	else:
		return False, [], False

def MatchIRF(pr_path, pr_irf, sec_file, svol):
	path = ""
        for sec_finfo in sec_file:
		if sec_finfo.tag == "path":
                        path = sec_finfo.text
                if sec_finfo.tag == "inheritedRightsFilter":
                        if sec_finfo.attrib.get("value") == pr_irf:
				logging.debug("Path: " + path + " Matching IRF: " + pr_irf + "\tStatus: OK")
                                return True
                        else:
				logging.info("IRF Mismatch Path: " + path + " Primary: " + pr_irf + " Secondary: " + sec_finfo.attrib.get("value"))
				if sync:
					SetIRF(svol+":"+path, pr_irf)
                                return True

def MatchTrusteeRights(pr_tname, pr_tid, pr_rights, sec_file,pr_type,svol):
	path = ""
	sec_tname = ""
	sec_rights = ""
        for sec_finfo in sec_file:
                trusteematch = False
                rightsmatch = False
		if sec_finfo.tag == "path":
			path = sec_finfo.text
                if sec_finfo.tag == "trustee":
                        for sec_trinfo in sec_finfo:
                                if sec_trinfo.tag == "name":
					trusteematch = False
					rightsmatch = False
					sec_tname = sec_trinfo.text
                                        if sec_trinfo.text == pr_tname:
                                                trusteematch = True
				if sec_tname == "Unknown(21350)" and sec_trinfo.tag == "id":
					if sec_trinfo.text == pr_tid:
						if unknown_display:
							logging.debug("Unknown Error: " + path + " Primary: " + pr_tname + " " + pr_rights + " Secondary: " + sec_tname + " " + sec_rights)
						trusteematch = True
				if trusteematch and sec_trinfo.tag == "rights":
					sec_rights = sec_trinfo.attrib.get("value")
					if sec_trinfo.attrib.get("value") == pr_rights:
						rightsmatch = True
						logging.debug("Path: " + path + " Matching Trustee: " + pr_tname + " " + pr_rights + "\tStatus: OK")
						return
					elif trusteematch:
						break
		if trusteematch:
			break
        if not rightsmatch:
		if not trusteematch:
			sec_tname = "None"
			sec_rights = ""
		logging.info("Trustee mis-match Path: " + path + " Primary: " + pr_tname + " " + pr_rights + " Secondary: " + sec_tname + " " + sec_rights)
		if sync:
			SetTrustee(svol+":"+path, pr_tname, pr_rights, pr_type)
		return 0



def ProcessTrusteeVerifySyncVol(pvol):
	cmd1 = ["ncpcon", "volume", pvol]
	buf1 = spr.Popen(cmd1, stdout=spr.PIPE, stderr=spr.PIPE).communicate()
  	if buf1[0].find("not found") > -1: 
		logging.error("Volume " +  pvol + " is not available")
		return
	st=buf1[0].find("Mount point:")+1+len("Mount point:")
	en=buf1[0].find("Shadow")-1
	ppath=buf1[0][st:en].strip()+"/"
	st1=buf1[0].find("Shadow Mount point:")+1+len("Shadow Mount point:")
	en1=buf1[0].find("Capacity")-1
	spath=buf1[0][st1:en1].strip()+"/"
	if spath == "(null)/": 
		logging.error("Volume " +  pvol + " doesn't have shadow volume!")
		return
	svol = spath.strip().split('/')[3]
	VerifyAndSyncRights(pvol,svol)
	return

def getVolumeMountPath(vol):
	mount_path_cmd = "nlvm list volume " + vol + " -t | grep 'Mountpoint=' | cut -d ' ' -f 7 | cut -d'=' -f2"
	p = spr.Popen(mount_path_cmd, shell=True, stdout=spr.PIPE, stderr=spr.PIPE, stdin=spr.PIPE)
	output = p.stdout.read()
	err = p.stderr.read()
	if not err == '':
		logging.error("Error Parsing TrusteeInfo of Volume: " + vol)
		sys.exit(0)
	return output.strip()



def VerifyAndSyncRights(privol,secvol):
	logging.info("Processing Primary Volume: " + privol + " Secondary Volume: " + secvol + " Sync ACL: " + str(sync))
        pr_trustee_info = ParseTrusteeInfo(privol)
        sr_trustee_info = ParseTrusteeInfo(secvol)
	sec_mountpath = getVolumeMountPath(secvol)
        for file in pr_trustee_info[0]:
                res = None
		res1 = None
                irf = ""
                path = ""
                for file_info in file:
                        if res == None and file_info.tag == "path":
                                path = file_info.text
                                res, sec_file, res1 = FindPath(sr_trustee_info, path, sec_mountpath)
				logging.debug("ACL for path " + path + " Found in Secondary:" + str(res or res1))
                        if res or res1:
                                if file_info.tag == "inheritedRightsFilter":
                                        irf = file_info.attrib.get("value")
                                        if res and irf != "":
                                                MatchIRF(path,irf,sec_file,secvol)
					elif res1 and irf != "":
						logging.info("IRF Mismatch Path: " + path + " Primary: " + irf + " Secondary: NA")
						if sync:
							SetIRF(secvol+":"+path, irf)
                                if file_info.tag == "trustee":
                                        tname = ""
                                        rights = ""
                                        type = "eDir"
                                        if file_info.attrib.get("type") != None:
                                                type = file_info.attrib.get("type")
                                        for trustee_info in file_info:
                                                if trustee_info.tag == "name":
                                                        tname = trustee_info.text
						if trustee_info.tag == "id":
							tid = trustee_info.text
                                                if trustee_info.tag == "rights":
                                                        rights = trustee_info.attrib.get("value")
					if tname != "Unknown(21350)" and tname != "" and tname != "" and rights != "":
						if res:
							MatchTrusteeRights(tname, tid, rights, sec_file,type,secvol)
						elif res1:
							logging.info("Trustee mis-match Path: " + path + " Primary: " + tname + " " + rights + " Secondary: NA")
							if sync:
								SetTrustee(secvol+":"+path, tname, rights, type)


def disphelp():
	print ""
	print "****************************HELP*************************************************"
	print "Option [-h or --help] displays this help screen"
	print "Option [-v or --verify] verifies if there are any the trustee rights differences between Primary and Shadow Volumes"
	print "Option [-s or --sync] syncs the trustee rights between Primary and Shadow Volumes"
	print "Option [-n or --ncpcon] inputs volumes from \"ncpcon volumes\" command for verify/sync"
	print "Option [-d or --debug] print debug logs"
	print "Option [-u or --unknown] print unknown error (false positives)"
	print "Option [-L or --LONG] ignore case when comparing  file name"
	print "Option [--logfile=] is to provide the log file name. Make sure that the log directory already exists"
	print "Option [--volumes=] is to provide a list of volumes on the command-line rather than from \"ncpcon volumes\""
	print "for verify/sync. This will be effective only when option [-n or --ncpcon] is not there."
	print ""
	print "Examples:- (1) To  verify and sync the trustee rights on the volume list from \"ncpcon volumes\""
	print "             ", sys.argv[0], " -vsn"
	print "           (2) To  verify and sync the trustee rights on the given volume list"
	print "             ", sys.argv[0], " -vs --volumes=VOL1 VOL2 VOL3"
	print "           (3) To  verify the trustee rights on the given volume list"
	print "             ", sys.argv[0], " -v --logfile=/var/log/dst_verify_resync_trustee.log --volumes=VOL1 VOL2 VOL3"
	print ""
	sys.exit(1)
	

reload(sys)
sys.setdefaultencoding('utf8')

excl = ["._DUPLICATE_FILES", "._NETWARE"]
volexcl = ["SYS", "_ADMIN"]
rites = [["supervisor", "s"], ["read", "r"], ["write", "w"], ["create", "c"], ["erase", "e"], ["access control", "a"], ["scan", "f"], ["modify", "m"]]
volumes = []
verify = False
sync = False
ncpcon = False
help = False
debug = False
unknown_display = False
long_name_space = False
logfile = ''
options = []

#print 'ARGV      :', sys.argv[1:]
try:
	options, remainder = getopt.gnu_getopt(sys.argv[1:], 'vsnhduL', ['verify', 
								       'sync',
								       'nss',
								       'help',
								       'debug',
									'unknown',
									 '--LONG',
									 'logfile=',
								       'volumes=',
								       ])
except:
	help = True
	disphelp()
#print 'OPTIONS   :', options
for opt, arg in options:
	if opt in ('-v', '--verify'):
		verify = True 
	elif opt in ('-s', '--sync'):
		sync = True
	elif opt in ('-n', '--nss'):
		ncpcon = True
	elif opt in ('-d', '--debug'):
		debug = True
	elif opt in ('-u', '--unknown'):
		unknown_display = True
	elif opt in ('-L', '--LONG'):
                long_name_space = True
	elif opt in ('--logfile'):
		logfile = arg
	elif opt == '--volumes':
		if not ncpcon:
			tmp = [arg]
			tmp.extend(remainder)
			volumes = tmp
	elif opt in ('-h', '--help'):
		help = True
		disphelp()
if logfile == '':
	logfile = sys.argv[0].split('.')[0] + '.log'

try:
	if debug:
		logging.basicConfig(filename=logfile, filemode='a', level=logging.DEBUG,format='%(asctime)s %(message)s', datefmt='%m/%d/%Y %I:%M:%S %p')
	else:
		logging.basicConfig(filename=logfile, filemode='a', level=logging.INFO,format='%(asctime)s %(message)s', datefmt='%m/%d/%Y %I:%M:%S %p')	
except IOError as e:
	print("Unable to create / Open log file " +  logfile)
	sys.exit(1)
	

if ncpcon:
	cmd0 = ["ncpcon", "volumes"]
	res = spr.Popen(cmd0, stdout=spr.PIPE, stderr=spr.PIPE).communicate()
	vst = res[0].find("Mounted Volumes")+len("Mounted Volumes")+2
	ven = res[0].find("volumes mounted")-4
	volumes = res[0][vst:ven].strip().split('\n')
	i = 0
	for vol in volumes:
		volumes[i] = vol.strip()
		i = i + 1
	volumes.remove(volexcl[0])
	volumes.remove(volexcl[1])

logging.info('----------------------------------------------------------------------------------')
logging.info('VERIFY:' + str(verify))
logging.info('SYNC: ' + str(sync))
logging.info('NCPCON: ' + str(ncpcon))
logging.info('DEBUG: ' + str(debug))
logging.info('LONG NAME SPACE: ' + str(long_name_space))
logging.info('VOLUMES: ' + str(volumes))
logging.debug('LOGFILE: ' + logfile)
#print 'REMAINING :', remainder
#argc = len(sys.argv)
#if argc < 2:
#	print "Error: inavlid arguments"
#	print "usage:", sys.argv[0], " volume names seperated by space"
#	quit()
for vol in volumes:
	logging.info("Processing Volume: " + vol.upper())
	ProcessTrusteeVerifySyncVol(vol.upper())
logging.info("Done with Trustee Processing!")
